@@ -1,12 +1,12 @@
 import re
+from datetime import date
+from decimal import Decimal
 from os import path
 
 import yaml
 from beancount.core import amount, data
 from beancount.core.number import D
 from beancount.ingest import importer
-from beancount.parser import options
-from beancount.query import query
 from ibflex import Types, client, parser
 from ibflex.enums import CashAction
 
@@ -22,7 +22,7 @@ class Importer(importer.ImporterProtocol):
     def file_account(self, file):
         return ""
 
-    def matches(self, trx, t):
+    def matches(self, trx, t, account):
         p = re.compile(r".* (?P<perShare>\d+\.?\d+) PER SHARE")
         trxPerShare = p.search(trx.description).group("perShare")
         tPerShare = p.search(t["description"]).group("perShare")
@@ -31,6 +31,7 @@ class Importer(importer.ImporterProtocol):
             t["date"] == trx.dateTime
             and t["symbol"] == trx.symbol
             and trxPerShare == tPerShare
+            and t["account"] == account
         )
 
     def extract(self, file, existing_entries):
@@ -45,13 +46,38 @@ class Importer(importer.ImporterProtocol):
         statement = parser.parse(response)
         assert isinstance(statement, Types.FlexQueryResponse)
 
+        result = []
         transactions = []
         for stmt in statement.FlexStatements:
+            account = stmt.accountId
+            for trx in stmt.Trades:
+                result.append(
+                    self.createBuy(
+                        trx.tradeDate,
+                        account,
+                        trx.symbol.rstrip("z"),
+                        trx.quantity,
+                        trx.currency,
+                        trx.tradePrice,
+                        amount.Amount(
+                            round(-trx.ibCommission, 2), trx.ibCommissionCurrency
+                        ),
+                        amount.Amount(round(trx.netCash, 2), trx.currency),
+                        config["baseCcy"],
+                        trx.fxRateToBase,
+                    )
+                )
+
             for trx in stmt.CashTransactions:
                 existingEntry = None
                 if CashAction.DIVIDEND == trx.type or CashAction.WHTAX == trx.type:
                     existingEntry = next(
-                        (t for t in transactions if self.matches(trx, t)), None
+                        (
+                            t
+                            for t in transactions
+                            if self.matches(trx, t, stmt.accountId)
+                        ),
+                        None,
                     )
 
                 if existingEntry:
@@ -63,10 +89,10 @@ class Importer(importer.ImporterProtocol):
                         existingEntry["type"] = trx.type
                 else:
                     if CashAction.WHTAX == trx.type:
-                        amount = 0
+                        amt = 0
                         whAmount = trx.amount
                     else:
-                        amount = trx.amount
+                        amt = trx.amount
                         whAmount = 0
 
                     transactions.append(
@@ -74,15 +100,14 @@ class Importer(importer.ImporterProtocol):
                             "date": trx.dateTime,
                             "symbol": trx.symbol,
                             "currency": trx.currency,
-                            "amount": amount,
+                            "amount": amt,
                             "whAmount": whAmount,
                             "description": trx.description,
                             "type": trx.type,
-                            "account": stmt.accountId,
+                            "account": account,
                         }
                     )
 
-            result = []
             for trx in transactions:
                 if trx["type"] == CashAction.DIVIDEND:
                     asset = trx["symbol"].rstrip("z")
@@ -93,53 +118,10 @@ class Importer(importer.ImporterProtocol):
                     currency = trx["currency"]
                     account = trx["account"]
 
-                    _, rows = query.run_query(
-                        existing_entries,
-                        options.OPTIONS_DEFAULTS,
-                        'select sum(number) as quantity, account where currency="'
-                        + asset
-                        + '" and date<#"'
-                        + str(payDate)
-                        + '" group by account;',
-                    )
-                    totalQuantity = D(0)
-                    for row in rows:
-                        totalQuantity += row.quantity
-
-                    remainingPayout = totalPayout
-                    remainingWithholding = totalWithholding
-                    for row in rows[:-1]:
-                        myAccount = row.account
-                        myQuantity = row.quantity
-
-                        myPayout = round(totalPayout * myQuantity / totalQuantity, 2)
-                        remainingPayout -= myPayout
-                        myWithholding = round(
-                            totalWithholding * myQuantity / totalQuantity, 2
-                        )
-                        remainingWithholding -= myWithholding
-                        result.append(
-                            self.createSingle(
-                                myPayout,
-                                myWithholding,
-                                myQuantity,
-                                myAccount,
-                                asset,
-                                currency,
-                                payDate,
-                                priceLookup,
-                                trx["description"],
-                                account,
-                            )
-                        )
-
-                    lastRow = rows[-1]
                     result.append(
-                        self.createSingle(
-                            remainingPayout,
-                            remainingWithholding,
-                            lastRow.quantity,
-                            lastRow.account,
+                        self.createDividen(
+                            totalPayout,
+                            totalWithholding,
                             asset,
                             currency,
                             payDate,
@@ -151,22 +133,21 @@ class Importer(importer.ImporterProtocol):
 
         return result
 
-    def createSingle(
+    def createDividen(
         self,
-        payout,
-        withholding,
-        quantity,
-        assetAccount,
-        asset,
-        currency,
-        date,
-        priceLookup,
-        description,
-        account,
+        payout: Decimal,
+        withholding: Decimal,
+        asset: str,
+        currency: str,
+        date: date,
+        priceLookup: PriceLookup,
+        description: str,
+        account: str,
     ):
-        narration = "Dividend for " + str(quantity) + " : " + description
-        liquidityAccount = self.getLiquidityAccount(assetAccount, asset, currency)
-        incomeAccount = self.getIncomeAccount(assetAccount, asset)
+        narration = "Dividend: " + description
+        liquidityAccount = self.getLiquidityAccount(account, currency)
+        incomeAccount = self.getIncomeAccount(account)
+        assetAccount = self.getAssetAccount(account, asset)
 
         price = priceLookup.fetchPrice(currency, date)
 
@@ -184,7 +165,7 @@ class Importer(importer.ImporterProtocol):
             ),
         ]
         if withholding > 0:
-            receivableAccount = self.getReceivableAccount(assetAccount, asset)
+            receivableAccount = self.getReceivableAccount(account)
             postings.append(
                 data.Posting(
                     receivableAccount,
@@ -202,15 +183,68 @@ class Importer(importer.ImporterProtocol):
             meta, date, "*", "", narration, data.EMPTY_SET, data.EMPTY_SET, postings
         )
 
-    def getLiquidityAccount(self, assetAccount, asset, currency):
-        return assetAccount.replace(":Investment:", ":Liquidity:").replace(
-            ":" + asset, ":" + currency
+    def createBuy(
+        self,
+        date: date,
+        account: str,
+        asset: str,
+        quantity: Decimal,
+        currency: str,
+        price: Decimal,
+        commission: amount.Amount,
+        netCash: amount.Amount,
+        baseCcy: str,
+        fxRateToBase: Decimal,
+    ):
+        narration = "Buy"
+        feeAccount = self.getFeeAccount(account)
+        liquidityAccount = self.getLiquidityAccount(account, currency)
+        assetAccount = self.getAssetAccount(account, asset)
+
+        liquidityPrice = None
+        if currency != baseCcy:
+            price = price * fxRateToBase
+            commission = amount.Amount(
+                round(commission.number * fxRateToBase, 2), baseCcy
+            )
+            liquidityPrice = amount.Amount(fxRateToBase, baseCcy)
+
+        postings = [
+            data.Posting(
+                assetAccount,
+                amount.Amount(quantity, asset),
+                data.Cost(price, baseCcy, None, None),
+                None,
+                None,
+                None,
+            ),
+            data.Posting(feeAccount, commission, None, None, None, None),
+            data.Posting(
+                liquidityAccount,
+                netCash,
+                None,
+                liquidityPrice,
+                None,
+                None,
+            ),
+        ]
+
+        meta = data.new_metadata("buy", 0, {"account": account})
+        return data.Transaction(
+            meta, date, "*", "", narration, data.EMPTY_SET, data.EMPTY_SET, postings
         )
 
-    def getReceivableAccount(self, assetAccount, asset):
-        parts = assetAccount.split(":")
-        return "Assets:" + parts[1] + ":Receivable:Verrechnungssteuer"
+    def getAssetAccount(self, account: str, asset: str):
+        return f"Asset:{account}:Investment:IB:{asset}"
 
-    def getIncomeAccount(self, assetAccount, asset):
-        parts = assetAccount.split(":")
-        return "Income:" + parts[1] + ":Interest"
+    def getLiquidityAccount(self, account: str, currency: str):
+        return f"Asset:{account}:Liquidity:IB:{currency}"
+
+    def getReceivableAccount(self, account: str):
+        return f"Assets:{account}:Receivable:Verrechnungssteuer"
+
+    def getIncomeAccount(self, account: str):
+        return f"Income:{account}:Interest"
+
+    def getFeeAccount(self, account: str):
+        return f"Expenses:{account}:Fees"
